@@ -1,7 +1,7 @@
 import numpy as np
 from warnings import warn
 import logging
-from scipy.integrate._ivp.rk import norm, SAFETY, MAX_FACTOR, MIN_FACTOR
+from scipy.integrate._ivp.rk import norm, MAX_FACTOR, MIN_FACTOR
 from extensisq.common import RungeKutta, HornerDenseOutput, NFS
 
 
@@ -66,6 +66,16 @@ class BS5(RungeKutta):
         of `nfev_stiff_detect`. For the assessment itself, the problem is
         assessed as non-stiff if the predicted nfev to complete the integration
         is lower than `nfev_stiff_detect`. The default value is 5000.
+    sc_params : tuple of size 4, "standard", "G", "H" or "W", optional
+        Parameters for the stepsize controller (k*b1, k*b2, a2, g). The step
+        size controller is, with k the exponent of the standard controller,
+        _n for new and _o for old:
+            h_n = h * g**(k*b1 + k*b2) * (h/h_o)**-a2
+                * (err/tol)**-b1 * (err_o/tol_o)**-b2
+        Predefined parameters are:
+            Gustafsson "G" (0.7, -0.4, 0, 0.9),  Watts "W" (2, -1, -1, 0.8),
+            Soederlind "S" (0.6, -0.2, 0, 0.9),  and "standard" (1, 0, 0, 0.9).
+        The default for this method is "S".
     interpolant : 'best', 'low' or 'free', optional
         Select the interpolant for dense output. The option 'best' is for the
         accurate fifth order interpolant described in [1], which needs 3 extra
@@ -79,37 +89,11 @@ class BS5(RungeKutta):
         is 'low': a safe option for which the method does not loose much of its
         performance when it is used with dense output.
 
-    Attributes
-    ----------
-    n : int
-        Number of equations.
-    status : string
-        Current status of the solver: 'running', 'finished' or 'failed'.
-    t_bound : float
-        Boundary time.
-    direction : float
-        Integration direction: +1 or -1.
-    t : float
-        Current time.
-    y : ndarray
-        Current state.
-    t_old : float
-        Previous time. None if no steps were made yet.
-    step_size : float
-        Size of the last successful step. None if no steps were made yet.
-    nfev : int
-        Number evaluations of the system's right-hand side.
-    njev : int
-        Number of evaluations of the Jacobian. Is always 0 for this solver as
-        it does not use the Jacobian.
-    nlu : int
-        Number of LU decompositions. Is always 0 for this solver.
-
     References
     ----------
     .. [1] P. Bogacki, L.F. Shampine, "An efficient Runge-Kutta (4,5) pair",
            Computers & Mathematics with Applications, Vol. 32, No. 6, 1996,
-           pp. 15-28, ISSN 0898-1221.
+           pp. 15-28.
            https://doi.org/10.1016/0898-1221(96)00141-1
     .. [2] RKSUITE: https://www.netlib.org/ode/rksuite/
     """
@@ -120,6 +104,7 @@ class BS5(RungeKutta):
     n_extra_stages = 3      # for dense output
     tanang = 5.2
     stbrad = 3.9
+    sc_params = "S"
 
     # time step fractions
     C = np.array([0, 1/6, 2/9, 3/7, 2/3, 3/4, 1])
@@ -224,10 +209,10 @@ class BS5(RungeKutta):
             -374785310/36476731, 186390855/36476731]])
 
     def __init__(self, fun, t0, y0, t_bound, nfev_stiff_detect=5000,
-                 interpolant='low', **extraneous):
+                 sc_params='standard', interpolant='low', **extraneous):
         super(BS5, self).__init__(
             fun, t0, y0, t_bound, nfev_stiff_detect=nfev_stiff_detect,
-            **extraneous)
+            sc_params=sc_params, **extraneous)
         # custom initialization to create extended storage for dense output
         if interpolant not in ('best', 'low', 'free'):
             raise ValueError(
@@ -258,8 +243,9 @@ class BS5(RungeKutta):
         h_abs = min(self.max_step, max(min_step, h_abs))
 
         # handle final integration steps
-        d = abs(self.t_bound - t)               # remaining interval
+        d = abs(self.t_bound - t)                          # remaining interval
         if d < 2 * h_abs:
+            self.standard_sc = True
             if d >= min_step:
                 if h_abs < d:
                     # h_abs < d < 2 * h_abs:
@@ -306,9 +292,13 @@ class BS5(RungeKutta):
             # reject step if pre_error too large
             if error_norm_pre > 1:
                 step_rejected = True
-                h_abs *= max(MIN_FACTOR,
-                             SAFETY * error_norm_pre ** self.error_exponent)
+                h_abs *= max(
+                    MIN_FACTOR,
+                    self.safety * error_norm_pre ** self.error_exponent)
+
                 NFS[()] += 1
+                if self.nfev_stiff_detect:
+                    self.jflstp += 1                  # for stiffness detection
                 continue
 
             # calculate next stage
@@ -320,26 +310,46 @@ class BS5(RungeKutta):
             # and evaluate
             if error_norm < 1:
                 step_accepted = True
-                if error_norm == 0:
-                    factor = MAX_FACTOR
+
+                # don't trust very small error_norm values
+                error_norm = max(self.min_error_norm, error_norm)
+
+                if self.standard_sc:
+                    factor = self.safety * error_norm ** self.error_exponent
+                    self.standard_sc = False
+
                 else:
-                    factor = min(MAX_FACTOR,
-                                 SAFETY * error_norm ** self.error_exponent)
+                    # use second order SC controller
+                    h_ratio = h / self.h_previous
+                    factor = self.safety_sc * (
+                        error_norm ** self.minbeta1 *
+                        self.error_norm_old ** self.minbeta2 *
+                        h_ratio ** self.minalpha)
+                    factor = min(MAX_FACTOR, max(MIN_FACTOR, factor))
+
                 if step_rejected:
                     factor = min(1, factor)
+
                 h_abs *= factor
 
             else:
+                if np.isnan(error_norm) or np.isinf(error_norm):
+                    return False, "Overflow or underflow encountered."
+
                 step_rejected = True
                 h_abs *= max(MIN_FACTOR,
-                             SAFETY * error_norm ** self.error_exponent)
+                             self.safety * error_norm ** self.error_exponent)
+
                 NFS[()] += 1
+                if self.nfev_stiff_detect:
+                    self.jflstp += 1                  # for stiffness detection
 
         # store for next step and interpolation
         self.h_previous = h
         self.y_old = y
         self.h_abs = h_abs
         self.f = self.K[self.n_stages].copy()
+        self.error_norm_old = error_norm
 
         # output
         self.t = t_new
@@ -348,7 +358,7 @@ class BS5(RungeKutta):
         # stiffness detection
         if self.nfev_stiff_detect:
             self.havg = 0.9 * self.havg + 0.1 * h     # exp moving average
-            self._stiff()
+            self._diagnose_stiffness()
             self.okstp += 1
             if self.okstp == 20:
                 self.havg = h
