@@ -1,10 +1,13 @@
 import numpy as np
 from warnings import warn
-from scipy.integrate._ivp.rk import norm, SAFETY
+from scipy.integrate._ivp.rk import norm
 from extensisq.common import (
     RungeKutta, HornerDenseOutput, CubicDenseOutput, LinearDenseOutput, NFS)
 
 
+SAFETY = 0.9
+
+
 class CK5(RungeKutta):
     """A 5th order method with 4th order error estimator that uses the
     coefficients of Cash and Karp [1]_.
@@ -59,6 +62,16 @@ class CK5(RungeKutta):
         of `nfev_stiff_detect`. For the assessment itself, the problem is
         assessed as non-stiff if the predicted nfev to complete the integration
         is lower than `nfev_stiff_detect`. The default value is 5000.
+    sc_params : tuple of size 4, "standard", "G", "H" or "W", optional
+        Parameters for the stepsize controller (k*b1, k*b2, a2, g). The step
+        size controller is, with k the exponent of the standard controller,
+        _n for new and _o for old:
+            h_n = h * g**(k*b1 + k*b2) * (h/h_o)**-a2
+                * (err/tol)**-b1 * (err_o/tol_o)**-b2
+        Predefined parameters are:
+            Gustafsson "G" (0.7, -0.4, 0, 0.9),  Watts "W" (2, -1, -1, 0.8),
+            Soederlind "S" (0.6, -0.2, 0, 0.9),  and "standard" (1, 0, 0, 0.9).
+        The default for this method is "G".
 
     References
     ----------
@@ -73,6 +86,7 @@ class CK5(RungeKutta):
     error_estimator_order = 4
     tanang = 2.4
     stbrad = 3.7
+    sc_params = "G"
 
     A = np.array([
         [0, 0, 0, 0, 0, 0],
@@ -114,7 +128,8 @@ class CKdisc(RungeKutta):
     evaluations.
 
     Step size is expected to be irregular in this method. This can interfere
-    with stiffness detection, which is therefore disabled.
+    with stiffness detection and non-standard stepsize control, which are
+    therefore disabled.
 
     Can be applied in the complex domain.
 
@@ -340,6 +355,8 @@ class CKdisc(RungeKutta):
                         break
 
                     # fifth order solution NOT accepted
+                    if np.isnan(E4) or np.isinf(E4):
+                        return False, "Overflow or underflow encountered."
 
                     # update twiddle factors
                     e = [E1, E2]
