@@ -5,29 +5,28 @@ import logging
 from scipy.integrate._ivp.common import (
     validate_max_step, norm, validate_first_step, warn_extraneous)
 from scipy.integrate._ivp.base import OdeSolver, DenseOutput
-from scipy.integrate._ivp.rk import SAFETY, MIN_FACTOR, MAX_FACTOR
+from scipy.integrate._ivp.rk import MIN_FACTOR, MAX_FACTOR
 
 
 NFS = np.array(0)                                         # failed step counter
 
 
 def validate_tol(rtol, atol, y):
-    """Validate tolerance values rtol and atol can be scalar or array-like.
-    The bound values are from RKSuite. These differ from those in scipy.
-    Bounds are applied without warning.
+    """Validate tolerance values. atol can be scalar or array-like, rtol a
+    scalar. The bound values are from RKSuite. These differ from those in
+    scipy. Bounds are applied without warning.
     """
     atol = np.asarray(atol)
-    rtol = np.asarray(rtol)
     if atol.ndim > 0 and atol.shape != (y.size, ):
         raise ValueError("`atol` has wrong shape.")
     if np.any(atol < 0):
         raise ValueError("`atol` must be positive.")
-    if rtol.ndim > 0 and rtol.shape != (y.size, ):
-        raise ValueError("`rtol` has wrong shape.")
-    if np.any(rtol < 0):
+    if not isinstance(rtol, float):
+        raise ValueError("`rtol` must be a float.")
+    if rtol < 0:
         raise ValueError("`rtol` must be positive.")
 
-    # atol cannot be exactly zero anymore.
+    # atol cannot be exactly zero.
     # For double precision float: sqrt(tiny) ~ 1.5e-154
     tiny = np.finfo(y.dtype).tiny
     atol = np.maximum(atol, sqrt(tiny))
@@ -57,6 +56,7 @@ class RungeKutta(OdeSolver):
       - the scale (weight) is smoothed differently
       - a different tolerance validation is used.
       - stiffness detection is added, can be turned off
+      - second order stepsize control is added.
     """
 
     # effective number of stages
@@ -84,11 +84,13 @@ class RungeKutta(OdeSolver):
     P: np.ndarray = NotImplemented              # shape: [n_stages + 1,
     #                                                     order_polynomial]
 
-    # Parameters for stiffness detection. These are not needed if stiffness
-    # detection is disabled (by setting nfev_stiff_detect=0).
+    # Parameters for stiffness detection, optional
     stbrad: float = NotImplemented              # radius of the arc
     tanang: float = NotImplemented              # tan(valid angle < pi/2)
 
+    # Parameters for stepsize control, optional
+    sc_params = NotImplemented                  # tuple, or str
+
     def _init_min_step_parameters(self):
         """Define the parameters h_min_a and h_min_b for the min_step rule:
             min_step = max(h_min_a * abs(t), h_min_b)
@@ -102,9 +104,8 @@ class RungeKutta(OdeSolver):
                 diff = abs(c1 - c2)
                 if diff:
                     cdiff = min(cdiff, diff)
-        small = 1e-3
-        if cdiff < small:
-            cdiff = small
+        if cdiff < 1e-3:
+            cdiff = 1e-3
             logging.warning(
                 'Some C-values of this Runge Kutta method are nearly the '
                 'same but not identical. This limits the minimum stepsize'
@@ -122,14 +123,48 @@ class RungeKutta(OdeSolver):
             raise ValueError(
                 "`nfev_stiff_detect` must be a non-negative integer.")
         self.nfev_stiff_detect = nfev_stiff_detect
-        if nfev_stiff_detect:
+        if NotImplemented in (self.stbrad, self.tanang):
+            # disable stiffness detection if not implemented
+            if nfev_stiff_detect not in (5000, 0):
+                warn("This method does not implement stiffness detection. "
+                     "Changing the value of nfev_stiff_detect does nothing.")
+            print("no stiffness detection")
+            self.nfev_stiff_detect = 0
+        if self.nfev_stiff_detect:
             self.okstp = 0                      # successful step counter
             self.jflstp = 0                     # failed step counter, last 40
             self.havg = 0.0                     # average stepsize
 
+    def _init_sc_control(self, sc_params):
+        coefs = {"G": (0.7, -0.4, 0, 0.9),
+                 "S": (0.6, -0.2, 0, 0.9),
+                 "W": (2, -1, -1, 0.8),
+                 "standard": (1, 0, 0, 0.9)}
+        if self.sc_params == NotImplemented:
+            # use standard controller if not specified otherwise
+            sc_params = sc_params or "standard"
+        else:
+            # use default controller of method if not specified otherwise
+            sc_params = sc_params or self.sc_params
+        if (isinstance(sc_params, str) and sc_params in coefs):
+            kb1, kb2, a, g = coefs[sc_params]
+        elif isinstance(sc_params, tuple) and len(sc_params) == 4:
+            kb1, kb2, a, g = sc_params
+        else:
+            raise ValueError('sc_params should be a tuple of length 3 or one '
+                             'of the strings "G", "S" or "standard"')
+        # set all parameters
+        self.minbeta1 = kb1 * self.error_exponent
+        self.minbeta2 = kb2 * self.error_exponent
+        self.minalpha = -a
+        self.safety = g
+        self.safety_sc = g ** (kb1 + kb2)
+        self.min_error_norm = (MAX_FACTOR/g) ** (1/self.error_exponent)
+        self.standard_sc = True                                # for first step
+
     def __init__(self, fun, t0, y0, t_bound, max_step=np.inf, rtol=1e-3,
                  atol=1e-6, vectorized=False, first_step=None,
-                 nfev_stiff_detect=5000, **extraneous):
+                 nfev_stiff_detect=5000, sc_params=None, **extraneous):
         warn_extraneous(extraneous)
         super(RungeKutta, self).__init__(fun, t0, y0, t_bound, vectorized,
                                          support_complex=True)
@@ -138,8 +173,10 @@ class RungeKutta(OdeSolver):
         self.f = self.fun(self.t, self.y)
         if self.f.dtype != self.y.dtype:
             raise TypeError('dtypes of solution and derivative do not match')
+        self.error_exponent = -1 / (self.error_estimator_order + 1)
         self._init_stiffness_detection(nfev_stiff_detect)
         self.h_min_a, self.h_min_b = self._init_min_step_parameters()
+        self._init_sc_control(sc_params)
 
         # size of first step:
         if first_step is None:
@@ -152,14 +189,13 @@ class RungeKutta(OdeSolver):
             self.h_abs = validate_first_step(first_step, t0, t_bound)
 
         self.K = np.empty((self.n_stages + 1, self.n), self.y.dtype)
-        self.error_exponent = -1 / (self.error_estimator_order + 1)
         self.FSAL = 1 if self.E[self.n_stages] else 0
         self.h_previous = None
         self.y_old = None
-        NFS[()] = 0                             # global failed step counter
+        NFS[()] = 0                                # global failed step counter
 
     def _step_impl(self):
-        # mostly follows the scipy implementation of RungeKutta
+        # mostly follows the scipy implementation of scipy's RungeKutta
         t = self.t
         y = self.y
 
@@ -171,6 +207,7 @@ class RungeKutta(OdeSolver):
         # handle final integration steps
         d = abs(self.t_bound - t)                          # remaining interval
         if d < 2 * h_abs:
+            self.standard_sc = True
             if d >= min_step:
                 if h_abs < d:
                     # h_abs < d < 2 * h_abs:
@@ -217,31 +254,49 @@ class RungeKutta(OdeSolver):
             if error_norm < 1:
                 step_accepted = True
 
-                if error_norm == 0:
-                    factor = MAX_FACTOR
+                # don't trust very small error_norm values
+                error_norm = max(self.min_error_norm, error_norm)
+
+                if self.standard_sc:
+                    factor = self.safety * error_norm ** self.error_exponent
+                    self.standard_sc = False
+
                 else:
-                    factor = min(MAX_FACTOR,
-                                 SAFETY * error_norm ** self.error_exponent)
+                    # use second order SC controller
+                    h_ratio = h / self.h_previous
+                    factor = self.safety_sc * (
+                        error_norm ** self.minbeta1 *
+                        self.error_norm_old ** self.minbeta2 *
+                        h_ratio ** self.minalpha)
+                    factor = min(MAX_FACTOR, max(MIN_FACTOR, factor))
+
                 if step_rejected:
                     factor = min(1, factor)
-                h_abs *= factor
 
-                if not self.FSAL:
-                    # evaluate ouput point for interpolation and next step
-                    self.K[self.n_stages] = self.fun(t + h, y_new)
+                h_abs *= factor
 
             else:
                 step_rejected = True
                 h_abs *= max(MIN_FACTOR,
-                             SAFETY * error_norm ** self.error_exponent)
-                NFS[()] += 1
-                self.jflstp += 1                      # for stiffness detection
+                             self.safety * error_norm ** self.error_exponent)
 
-        # store for next step and interpolation
+                NFS[()] += 1
+                if self.nfev_stiff_detect:
+                    self.jflstp += 1                  # for stiffness detection
+
+                if np.isnan(error_norm) or np.isinf(error_norm):
+                    return False, "Overflow or underflow encountered."
+
+        if not self.FSAL:
+            # evaluate ouput point for interpolation and next step
+            self.K[self.n_stages] = self.fun(t + h, y_new)
+
+        # store for next step, interpolation and stepsize control
         self.h_previous = h
         self.y_old = y
         self.h_abs = h_abs
         self.f = self.K[self.n_stages].copy()
+        self.error_norm_old = error_norm
 
         # output
         self.t = t_new
@@ -250,12 +305,13 @@ class RungeKutta(OdeSolver):
         # stiffness detection
         if self.nfev_stiff_detect:
             self.okstp += 1
-            self.havg = 0.9 * self.havg + 0.1 * h     # exp moving average
-            self._stiff()
+            self.havg = 0.9 * self.havg + 0.1 * h          # exp moving average
+            self._diagnose_stiffness()
+
+            # reset after the first 20 steps to:
+            # - get stepsize on scale
+            # - reduce the effect of a possible initial transient
             if self.okstp == 20:
-                # reset after the first 10 steps to:
-                # - get stepsize on scale
-                # - reduce the effect of a possible initial transient
                 self.havg = h
                 self.jflstp = 0
 
@@ -300,7 +356,7 @@ class RungeKutta(OdeSolver):
         Q = self.K.T @ self.P
         return HornerDenseOutput(self.t_old, self.t, self.y_old, Q)
 
-    def _stiff(self):
+    def _diagnose_stiffness(self):
         """Stiffness detection.
 
         Test only if there are many recent step failures, or after many
@@ -315,7 +371,7 @@ class RungeKutta(OdeSolver):
         # failures occurred in the last 40 successful steps.
         if self.okstp % 40 == 39:
             lotsfl = self.jflstp >= 10
-            self.jflstp = 0               # reset each time
+            self.jflstp = 0                               # reset each 40 steps
         else:
             lotsfl = False
 
@@ -348,7 +404,6 @@ class RungeKutta(OdeSolver):
             # inform the user about stiffness with warning messages
             # the messages about remaining work have been removed from the
             # original code.
-
             if stif is None:
                 # unsure about stiffness
                 if rootre is None:
@@ -391,18 +446,16 @@ class RungeKutta(OdeSolver):
                          'change to a code intended for stiff problems.')
             else:
                 # stif == False
+                # no warning is given
                 if rootre is None:
-                    # no warning is given
                     logging.info(
                         'Stiffness detection has diagnosed the problem as '
                         'non-stiff, without performing power iterations')
                 elif rootre:
-                    # no warning is given
                     logging.info(
                         'The problem has a real dominant root '
                         'and is not stiff')
                 else:
-                    # no warning is given
                     logging.info(
                         'The problem has a complex pair of dominant roots '
                         'and is not stiff')
