@@ -55,6 +55,16 @@ class Pr7(RungeKutta):
         of `nfev_stiff_detect`. For the assessment itself, the problem is
         assessed as non-stiff if the predicted nfev to complete the integration
         is lower than `nfev_stiff_detect`. The default value is 5000.
+    sc_params : tuple of size 4, "standard", "G", "H" or "W", optional
+        Parameters for the stepsize controller (k*b1, k*b2, a2, g). The step
+        size controller is, with k the exponent of the standard controller,
+        _n for new and _o for old:
+            h_n = h * g**(k*b1 + k*b2) * (h/h_o)**-a2
+                * (err/tol)**-b1 * (err_o/tol_o)**-b2
+        Predefined parameters are:
+            Gustafsson "G" (0.7, -0.4, 0, 0.9),  Watts "W" (2, -1, -1, 0.8),
+            Soederlind "S" (0.6, -0.2, 0, 0.9),  and "standard" (1, 0, 0, 0.9).
+        The default for this method is "S".
 
     References
     ----------
@@ -69,6 +79,7 @@ class Pr7(RungeKutta):
     n_stages = 10
     tanang = 7.5
     stbrad = 4.1
+    sc_params = "S"
 
     C = np.array([0, 1/6, 1/4, 1/2, 1/2, 3/16, 3/16, 3/5, 6/7, 1])
 
@@ -167,6 +178,16 @@ class Pr8(RungeKutta):
         of `nfev_stiff_detect`. For the assessment itself, the problem is
         assessed as non-stiff if the predicted nfev to complete the integration
         is lower than `nfev_stiff_detect`. The default value is 5000.
+    sc_params : tuple of size 4, "standard", "G", "H" or "W", optional
+        Parameters for the stepsize controller (k*b1, k*b2, a2, g). The step
+        size controller is, with k the exponent of the standard controller,
+        _n for new and _o for old:
+            h_n = h * g**(k*b1 + k*b2) * (h/h_o)**-a2
+                * (err/tol)**-b1 * (err_o/tol_o)**-b2
+        Predefined parameters are:
+            Gustafsson "G" (0.7, -0.4, 0, 0.9),  Watts "W" (2, -1, -1, 0.8),
+            Soederlind "S" (0.6, -0.2, 0, 0.9),  and "standard" (1, 0, 0, 0.9).
+        The default for this method is "G".
 
     References
     ----------
@@ -181,6 +202,7 @@ class Pr8(RungeKutta):
     n_stages = 13
     tanang = 6.0
     stbrad = 4.5
+    sc_params = "G"
 
     A = np.array([
         [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
@@ -397,6 +419,16 @@ class Pr9(RungeKutta):
         of `nfev_stiff_detect`. For the assessment itself, the problem is
         assessed as non-stiff if the predicted nfev to complete the integration
         is lower than `nfev_stiff_detect`. The default value is 5000.
+    sc_params : tuple of size 4, "standard", "G", "H" or "W", optional
+        Parameters for the stepsize controller (k*b1, k*b2, a2, g). The step
+        size controller is, with k the exponent of the standard controller,
+        _n for new and _o for old:
+            h_n = h * g**(k*b1 + k*b2) * (h/h_o)**-a2
+                * (err/tol)**-b1 * (err_o/tol_o)**-b2
+        Predefined parameters are:
+            Gustafsson "G" (0.7, -0.4, 0, 0.9),  Watts "W" (2, -1, -1, 0.8),
+            Soederlind "S" (0.6, -0.2, 0, 0.9),  and "standard" (1, 0, 0, 0.9).
+        The default for this method is "standard".
 
     References
     ----------
@@ -411,6 +443,7 @@ class Pr9(RungeKutta):
     n_stages = 17
     tanang = 7.5
     stbrad = 4.6
+    sc_params = "standard"
 
     A = np.array([
         [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
